@@ -22,11 +22,8 @@
 #include <time.h>
 #include <errno.h>
 #include <limits.h>
-#include <linux/unistd.h>
 
-#include <sys/prctl.h>
 #include <sys/stat.h>
-#include <sys/sysinfo.h>
 #include <sys/types.h>
 #include <sys/time.h>
 #include <sys/resource.h>
@@ -320,8 +317,10 @@ static int raise_soft_prio(int policy, const struct sched_param *param)
 		return err;
 	}
 
-	soft_max = (rlim.rlim_cur == RLIM_INFINITY) ? policy_max : rlim.rlim_cur;
-	hard_max = (rlim.rlim_max == RLIM_INFINITY) ? policy_max : rlim.rlim_max;
+	soft_max = (rlim.rlim_cur == RLIM_INFINITY) ? (unsigned int)policy_max
+						    : rlim.rlim_cur;
+	hard_max = (rlim.rlim_max == RLIM_INFINITY) ? (unsigned int)policy_max
+						    : rlim.rlim_max;
 
 	if (prio > soft_max && prio <= hard_max) {
 		rlim.rlim_cur = prio;
@@ -1617,7 +1616,7 @@ static void rstat_print_stat(struct thread_param *par, int index, int verbose, i
  * thread that creates a named fifo and hands out run stats when someone
  * reads from the fifo.
  */
-static void *fifothread(void *param)
+static void *fifothread(void *param __attribute__ ((unused)))
 {
 	int ret;
 	int fd;
@@ -1798,10 +1797,11 @@ rstat_err:
 	return;
 }
 
-static void write_stats(FILE *f, void *data)
+static void write_stats(FILE *f, void *data __attribute__ ((unused)))
 {
 	struct thread_param **par = parameters;
-	unsigned int i, j, comma;
+	int i, j;
+	unsigned comma;
 	struct thread_stat *s;
 
 	fprintf(f, "  \"num_threads\": %d,\n", num_threads);
@@ -1953,10 +1953,8 @@ int main(int argc, char **argv)
 		for (k=0; k < times; k++)
 			clock_gettime(clock, &time[k]);
 
-		if (ct_debug) {
-			info("For %d consecutive calls to clock_gettime():\n", times);
-			info("time, delta time (nsec)\n");
-		}
+		info(ct_debug, "For %d consecutive calls to clock_gettime():\n", times);
+		info(ct_debug, "time, delta time (nsec)\n");
 
 		prev = time[0];
 		for (k=1; k < times; k++) {
@@ -1967,10 +1965,9 @@ int main(int argc, char **argv)
 			if (diff && (diff < min_non_zero_diff))
 				min_non_zero_diff = diff;
 
-			if (ct_debug)
-				info("%ld.%06ld  %5llu\n",
-				     time[k].tv_sec, time[k].tv_nsec,
-				     (unsigned long long)diff);
+			info(ct_debug, "%ld.%06ld  %5llu\n",
+					time[k].tv_sec, time[k].tv_nsec,
+					(unsigned long long)diff);
 		}
 
 		free(time);
