@@ -65,7 +65,7 @@ struct State {
 struct State *statep;
 
 const int policy = SCHED_FIFO;
-const int prio_min;	/* Initialized for the minimum priority of policy */
+static int prio_min;	/* Initialized for the minimum priority of policy */
 
 static void usage(int error)
 {
@@ -114,7 +114,7 @@ int main(int argc, char *argv[])
 	mptr += sizeof(pthread_mutex_t);	/* advance the memory pointer */
 
 	/* Initialize our mutex via the resource pointer */
-	init_shared_pthread_mutex(resource, PTHREAD_PRIO_INHERIT, policy);
+	init_shared_pthread_mutex(resource, PTHREAD_PRIO_INHERIT);
 
 	statep = (struct State*)mptr;
 	mptr += sizeof(struct State);
@@ -125,7 +125,7 @@ int main(int argc, char *argv[])
 	mptr += sizeof(pthread_mutex_t);	/* advance the memory pointer */
 
 	/* Initialize our State mutex */
-	init_shared_pthread_mutex(statep->mutex, PTHREAD_PRIO_NONE, policy);
+	init_shared_pthread_mutex(statep->mutex, PTHREAD_PRIO_NONE);
 
 	set_rt_prio(0, prio_min, policy);
 
@@ -254,7 +254,7 @@ long process_shared_mutex_available(void)
 	res = sysconf(_SC_THREAD_PROCESS_SHARED);
 	if (res == -1) {
 		int err = errno;	/* save the error number */
-		err_msg("%s: sysconf(_SC_THREAD_PROCESS_SHARED): ");
+		err_msg("%s: sysconf(_SC_THREAD_PROCESS_SHARED): ", __func__);
 		err_exit(err, NULL);
 	}
 #else
@@ -323,7 +323,7 @@ void Pthread_mutex_unlock(pthread_mutex_t *mutex)
 	}
 }
 
-void init_shared_pthread_mutex(pthread_mutex_t *mutex, int protocol, int policy)
+void init_shared_pthread_mutex(pthread_mutex_t *mutex, int protocol)
 {
 	pthread_mutexattr_t attr;
 
