@@ -80,7 +80,13 @@ struct sched_data {
 };
 
 static int shutdown;
-
+static int info_enable;
+static int debug_enable;
+static int tracelimit;
+static int trace_marker;
+static pthread_mutex_t break_thread_id_lock = PTHREAD_MUTEX_INITIALIZER;
+static pid_t break_thread_id;
+static uint64_t break_thread_value;
 static pthread_barrier_t barrier;
 
 static int cpu_count;
@@ -358,6 +364,32 @@ static int mounted(const char *path, long magic)
 #define CGROUP_PATH "/sys/fs/cgroup"
 #define CPUSET_PATH CGROUP_PATH "/cpuset"
 
+/**
+ * cgroup_mounted - test if the path /sys/fs/cgroup exists
+ * and is a supported type
+ *
+ * Returns -1 if the path does not exist
+ * Returns 0 if the path exists but is not a cgroup type
+ * Returns 1 if the path exists and supports cgroups
+ */
+static int cgroup_mounted(void)
+{
+	int ret;
+
+	ret = mounted(CGROUP_PATH, TMPFS_MAGIC);
+	if (ret == -1)
+		return -1;	/* path doesn't exist */
+	if (ret == 1)
+		return 1;	/* tmpfs */
+	ret = mounted(CGROUP_PATH, CGROUP_SUPER_MAGIC);
+	if (ret == 1)
+		return 1;	/* cgroup v1 */
+	ret = mounted(CGROUP_PATH, CGROUP2_SUPER_MAGIC);
+	if (ret == 1)
+		return 1;	/* cgroup v2 */
+	return 0;	/* path exists but type is not recognized */
+}
+
 static int open_cpuset(const char *path, const char *name)
 {
 	char buf[MAXPATH];
@@ -383,14 +415,13 @@ static int mount_cpuset(void)
 	int fd;
 
 	/* Check if cgroups is already mounted. */
-	ret = mounted(CGROUP_PATH, TMPFS_MAGIC);
-	if (ret < 0)
+	ret = cgroup_mounted();
+	if (ret < 0)	/* /sys/fs/cgroup doesn't exist */
 		return ret;
-	if (!ret) {
-		ret = mount("cgroup_root", CGROUP_PATH, "tmpfs", 0, NULL);
-		if (ret < 0)
-			return ret;
-	}
+
+	if (!ret) /* /sys/fs/cgroup exists, but we don't recognize the type */
+		return -1;
+
 	ret = stat(CPUSET_PATH, &st);
 	if (ret < 0) {
 		ret = mkdir(CPUSET_PATH, 0755);
@@ -642,6 +673,10 @@ static void teardown(void)
 
 	destroy_cpuset(CPUSET_ALL, 0);
 	destroy_cpuset(CPUSET_LOCAL, 1);
+
+	/* close any tracer file descriptors */
+	disable_trace_mark();
+
 }
 
 static void usage(int error)
@@ -652,7 +687,6 @@ static void usage(int error)
 	       "-a [CPUSET] --affinity     Comma/hyphen separated list of CPUs to run deadline\n"
 	       "                           tasks on. An empty CPUSET runs on all CPUs a deadline\n"
 	       "                           task.\n"
-	       "                           on CPU 4, and thread #5 on CPU 5.\n"
 	       "-D TIME  --duration        Specify a length for the test run.\n"
 	       "                           Append 'm', 'h', or 'd' to specify minutes, hours or\n"
 	       "                           days\n"
@@ -664,6 +698,10 @@ static void usage(int error)
 	       "                           (default 500us).\n"
 	       "-t NUM   --threads         The number of threads to run as deadline (default 1).\n"
 	       "-q       --quiet           print a summary only on exit\n"
+	       "-b USEC  --breaktrace=USEC send break trace command when latency > USEC\n"
+	       "         --tracemark       write a trace mark when -b latency is exceeded\n"
+		   "		 --debug		   Print debugging info for cyclicdeadline\n"
+		   "		 --verbose		   Print useful information about the test\n"
 	       );
 	exit(error);
 }
@@ -760,8 +798,8 @@ void *run_deadline(void *data)
 	u64 period;
 	int ret;
 
-	printf("deadline thread %ld\n", tid);
-
+	debug(debug_enable, "deadline thread %ld\n", tid);
+	// set up for each measurment thread
 	stat->tid = tid;
 
 	ret = sched_getattr(0, &attr, sizeof(attr), 0);
@@ -773,17 +811,13 @@ void *run_deadline(void *data)
 		return NULL;
 	}
 
-	pthread_barrier_wait(&barrier);
-
 	attr.sched_policy = SCHED_DEADLINE;
 	attr.sched_runtime = sd->runtime_us * 1000;
 	attr.sched_deadline = sd->deadline_us * 1000;
 
-	printf("thread[%d] runtime=%lldus deadline=%lldus\n",
+	debug(debug_enable, "thread[%d] runtime=%lldus deadline=%lldus\n",
 	      gettid(), sd->runtime_us, sd->deadline_us);
 
-	pthread_barrier_wait(&barrier);
-
 	ret = sched_setattr(0, &attr, 0);
 	if (ret < 0) {
 		err_msg_n(errno, "[%ld]", tid);
@@ -800,6 +834,18 @@ void *run_deadline(void *data)
 
 	while (!shutdown) {
 		period = do_runtime(tid, sd, period);
+		if (tracelimit && (stat->max > tracelimit)) {
+			shutdown++;
+			pthread_mutex_lock(&break_thread_id_lock);
+			if (break_thread_id == 0) {
+				break_thread_id = stat->tid;
+				break_thread_value = stat->max;
+				tracemark("hit latency threshold (%lld > %d)",
+						 (unsigned long long) stat->max, tracelimit);
+			}
+			pthread_mutex_unlock(&break_thread_id_lock);
+			break;
+		}
 		sched_yield();
 	}
 	ret = sched_getattr(0, &attr, sizeof(attr), 0);
@@ -984,7 +1030,7 @@ static int calc_nr_cpus(const char *setcpu, char **buf)
 	return -1;
 }
 
-static void sighand(int sig)
+static void sighand(int sig __attribute__ ((unused)))
 {
 	shutdown = 1;
 }
@@ -1021,7 +1067,7 @@ static void write_stats(FILE *f, void *data)
 {
 	struct sched_data *sd = data;
 	struct thread_stat *s;
-	unsigned int i;
+	int i;
 
 	fprintf(f, "  \"num_threads\": %d,\n", nr_threads);
 	fprintf(f, "  \"resolution_in_ns\": %u,\n", use_nsecs);
@@ -1038,9 +1084,10 @@ static void write_stats(FILE *f, void *data)
 	fprintf(f, "  }\n");
 }
 
-enum options_valud {
+enum options_values {
 	OPT_AFFINITY=1, OPT_DURATION, OPT_HELP, OPT_INTERVAL,
-	OPT_JSON, OPT_STEP, OPT_THREADS, OPT_QUIET
+	OPT_JSON, OPT_STEP, OPT_THREADS, OPT_QUIET,
+	OPT_BREAKTRACE, OPT_TRACEMARK, OPT_INFO, OPT_DEBUG,
 };
 
 int main(int argc, char **argv)
@@ -1079,9 +1126,13 @@ int main(int argc, char **argv)
 			{ "step",	required_argument,	NULL,	OPT_STEP },
 			{ "threads",	required_argument,	NULL,	OPT_THREADS },
 			{ "quiet",	no_argument,		NULL,	OPT_QUIET },
+			{ "breaktrace",       required_argument, NULL, OPT_BREAKTRACE },
+			{ "tracemark",	     no_argument,	NULL, OPT_TRACEMARK },
+			{ "verbose",	no_argument,	NULL,	OPT_INFO},
+			{ "debug",	no_argument, 	NULL, 	OPT_DEBUG},
 			{ NULL,		0,			NULL,	0   },
 		};
-		c = getopt_long(argc, argv, "a::c:D:hi:s:t:q", options, NULL);
+		c = getopt_long(argc, argv, "a::c:D:hi:s:t:b:q", options, NULL);
 		if (c == -1)
 			break;
 		switch (c) {
@@ -1116,6 +1167,10 @@ int main(int argc, char **argv)
 		case 'D':
 			duration = parse_time_string(optarg);
 			break;
+		case 'b':
+		case OPT_BREAKTRACE:
+			tracelimit = atoi(optarg);
+			break;
 		case OPT_QUIET:
 		case 'q':
 			quiet = 1;
@@ -1124,6 +1179,15 @@ int main(int argc, char **argv)
 		case 'h':
 			usage(0);
 			break;
+		case OPT_TRACEMARK:
+			trace_marker = 1;
+			break;
+		case OPT_INFO:
+			info_enable = 1;
+			break;
+		case OPT_DEBUG:
+			debug_enable = 1;
+			break;
 		default:
 			usage(1);
 		}
@@ -1161,6 +1225,8 @@ int main(int argc, char **argv)
 		warn("mlockall");
 
 	setup_ftrace_marker();
+	if (tracelimit && trace_marker)
+		enable_trace_mark();
 
 	thread = calloc(nr_threads, sizeof(*thread));
 	sched_data = calloc(nr_threads, sizeof(*sched_data));
@@ -1175,7 +1241,7 @@ int main(int argc, char **argv)
 		percent = nr_cpus * 80 / nr_threads;
 	}
 
-	/* Set up the data while sill in SCHED_FIFO */
+	/* Set up the data while still in SCHED_FIFO */
 	for (i = 0; i < nr_threads; i++) {
 		sd = &sched_data[i];
 		/*
@@ -1196,7 +1262,7 @@ int main(int argc, char **argv)
 		sd->runtime_us = runtime;
 		sd->deadline_us = interval;
 
-		printf("interval: %lld:%lld\n", sd->runtime_us, sd->deadline_us);
+		info(info_enable, "interval: %lld:%lld\n", sd->runtime_us, sd->deadline_us);
 
 		/* Make sure that we can make our deadlines */
 		start_period = get_time_us();
@@ -1206,7 +1272,7 @@ int main(int argc, char **argv)
 			fatal("Failed to perform task within runtime: Missed by %lld us\n",
 			      end_period - start_period - sd->runtime_us);
 
-		printf("  Tested at %lldus of %lldus\n",
+		info(info_enable, "  Tested at %lldus of %lldus\n",
 		       end_period - start_period, sd->runtime_us);
 
 		interval += step;
@@ -1222,8 +1288,6 @@ int main(int argc, char **argv)
 
 	atexit(teardown);
 
-	pthread_barrier_wait(&barrier);
-
 	if (shutdown)
 		fatal("failed to setup child threads at step 1\n");
 
@@ -1251,9 +1315,7 @@ int main(int argc, char **argv)
 		system("cat /sys/fs/cgroup/cpuset/my_cpuset/tasks");
 	}
 
-	printf("main thread %d\n", gettid());
-
-	pthread_barrier_wait(&barrier);
+	debug(debug_enable, "main thread %d\n", gettid());
 
 	if (shutdown)
 		fatal("failed to setup child threads at step 2");
@@ -1269,6 +1331,13 @@ int main(int argc, char **argv)
 
 	loop(sched_data, nr_threads);
 
+	if (tracelimit) {
+		if (break_thread_id) {
+			printf("# Break thread: %d\n", break_thread_id);
+			printf("# Break value: %llu\n", (unsigned long long)break_thread_value);
+		}
+	}
+
 	for (i = 0; i < nr_threads; i++) {
 
 		sd = &sched_data[i];
