@@ -369,7 +369,7 @@ static void process_options(int argc, char *argv[])
 		if (duration < 0)
 			error = 1;
 
-		if (num_threads < 1 || num_threads > 255)
+		if (num_threads < 1)
 			error = 1;
 
 		if (priority < 0 || priority > 99)
@@ -384,13 +384,13 @@ static void process_options(int argc, char *argv[])
 
 static int volatile mustshutdown;
 
-static void sighand(int sig)
+static void sighand(int sig __attribute__ ((unused)))
 {
 	mustshutdown = 1;
 }
 
 static void print_stat(FILE *fp, struct params *receiver, struct params *sender,
-		       int verbose, int quiet)
+		       int verbose __attribute__ ((unused)), int quiet)
 {
 	int i;
 
@@ -435,7 +435,7 @@ static void write_stats(FILE *f, void *data)
 {
 	struct params_stats *ps = data;
 	struct params *s, *r;
-	unsigned int i;
+	int i;
 
 	fprintf(f, "  \"num_threads\": %d,\n", num_threads);
 	fprintf(f, "  \"thread\": {\n");
